using System;
using System.Collections.Generic;
using System.Text;
using System.ComponentModel;
using System.Windows.Forms;
using System.Collections;

namespace GGC_Sample1
{
    // Binding Object
    public class MyTable : BindingList<MyRow>, ITypedList
    {
        #region ITypedList Members

        public PropertyDescriptorCollection GetItemProperties(PropertyDescriptor[] listAccessors)
        {
            List<PropertyDescriptor> newList = new List<PropertyDescriptor>();
            PropertyDescriptorCollection coll = TypeDescriptor.GetProperties(typeof(MyRow));
            foreach (PropertyDescriptor prop in coll)
            {
                if (prop.IsBrowsable)
                    newList.Add(prop);
            }
            return new PropertyDescriptorCollection(newList.ToArray());
        }

        public string GetListName(PropertyDescriptor[] listAccessors)
        {
            return "MyTable";
        }

        protected override object AddNewCore()
        {
            //object o = base.AddNewCore();
            //MessageBox.Show("Item Count in BindingList: " + this.Count.ToString());
            //return o;

            MyRow setting = new MyRow(true);
            setting.Parent = this;
            AddingNewEventArgs e = new AddingNewEventArgs(setting);
            this.OnAddingNew(e);
            base.Add(setting);
            return setting;


        }

        #endregion
    }

    public class MyRow : IEditableObject
    {
        private MyTable m_Parent;

        public MyRow()
        {
            m_FirstName = String.Empty;
            m_LastName = String.Empty;
            m_DataObject = new CustomObject();
        }

        public MyRow(string firstName, string lastName, CustomObject dataObj)
        {
            m_FirstName = firstName;
            m_LastName = lastName;
            m_DataObject = dataObj;
        }

        public MyRow(bool isNew) : this()
        {
            m_IsAddNew = isNew;
        }

        [Browsable(false)]
        public MyTable Parent
        {
            get
            {
                return m_Parent;
            }
            set
            {
                m_Parent = value;
            }
        }

        private string m_FirstName;

        public string FirstName
        {
            get { return m_FirstName; }
            set { m_FirstName = value; }
        }

        private string m_LastName;

        public string LastName
        {
            get { return m_LastName; }
            set { m_LastName = value; }
        }

        private CustomObject m_DataObject;

        public CustomObject DataObject
        {
            get { return m_DataObject; }
            set { m_DataObject = value; }
        }

        #region IEditableObject Members

        private Hashtable m_OriginalData;
        private bool m_Editing = false;
        private bool m_IsAddNew = false;

        void IEditableObject.BeginEdit()
        {
            if (!m_Editing)
            {
                m_Editing = true;
                m_OriginalData = new Hashtable();
                PropertyDescriptorCollection Properties = TypeDescriptor.GetProperties(this);
                foreach (PropertyDescriptor prop in Properties)
                {
                    m_OriginalData.Add(prop, prop.GetValue(this));
                }
            }
        }

        void IEditableObject.CancelEdit()
        {
            if (m_Editing)
            {
                if ((m_IsAddNew))
                {
                    if (Parent != null)
                    {
                        int index = Parent.IndexOf(this);
                        Parent.CancelNew(index);
                    }
                }
                else
                {
                    PropertyDescriptor prop;
                    foreach (DictionaryEntry entry in m_OriginalData)
                    {
                        prop = (PropertyDescriptor)entry.Key;
                        prop.SetValue(this, entry.Value);
                    }
                    m_OriginalData = null;
                    m_Editing = false;
                }
            }
        }

        void IEditableObject.EndEdit()
        {
            if (m_Editing)
            {
                if ((m_IsAddNew))
                {

                    if (Parent != null)
                    {
                        int index = Parent.IndexOf(this);
                        Parent.EndNew(index);
                    }
                }
                m_OriginalData = null;
                m_Editing = false;
            }
        }

        #endregion

    }

    public interface ICustomObject
    {
        double DoubleValue
        {
            get;
            set;
        }
    }

    public class CustomObject : ICustomObject
    {
        public CustomObject(double value)
        {
            m_DoubleValue = value;
        }

        public CustomObject()
        {
            m_DoubleValue = 0;
        }

        private double m_DoubleValue;

        public double DoubleValue
        {
            get { return m_DoubleValue; }
            set { m_DoubleValue = value; }
        }

        public override string ToString()
        {
            return m_DoubleValue.ToString();
        }
    }
}
